-- ----------------------------
-- Table structure for dc_column_info
-- ----------------------------
DROP TABLE IF EXISTS `dc_column_info`;
CREATE TABLE `dc_column_info` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `table_cname` varchar(32) NOT NULL COMMENT '表中文名',
  `table_ename` varchar(32) NOT NULL COMMENT '表英文名',
  `column_cname` varchar(32) NOT NULL COMMENT '字段中文名',
  `column_ename` varchar(32) NOT NULL COMMENT '字段英文名',
  `type` varchar(2) NOT NULL DEFAULT '1' COMMENT '字段类型说明,1:普通字段，2:编号字段',
  `is_deleted` char(1) NOT NULL DEFAULT 'N' COMMENT '是否删除,N:未删除，Y:删除',
  `gmt_create` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录创建时间',
  `gmt_modified` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录修改时间,如果时间是1970年则表示纪录未修改',
  `creator` varchar(32) NOT NULL DEFAULT '0' COMMENT '创建人，0表示无创建人值',
  `modifier` varchar(32) NOT NULL DEFAULT '0' COMMENT '修改人,如果为0则表示纪录未修改',
  `remark` varchar(128) DEFAULT NULL COMMENT '备注',
  PRIMARY KEY (`id`),
  UNIQUE KEY `uniq_table_column_name` (`table_ename`,`column_ename`) USING BTREE COMMENT '字段table_ename和字段column_ename的值组合必须全表唯一'
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='数据库表中的列信息记录表';

-- ----------------------------
-- Table structure for dc_column_result
-- ----------------------------
DROP TABLE IF EXISTS `dc_column_result`;
CREATE TABLE `dc_column_result` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `hospital_id` bigint(20) NOT NULL DEFAULT '0' COMMENT '组织机构ID',
  `behospital_code` varchar(32) NOT NULL COMMENT '病人住院序号',
  `hospital_data_code` varchar(128) NOT NULL COMMENT '医院数据编号',
  `hosptial_datatmp_code` varchar(32) DEFAULT NULL COMMENT '医院病历文书模板编号/非文书模块编号',
  `hosptial_datatmp_name` varchar(32) DEFAULT NULL COMMENT '医院病历文书模板名称/非文书模块名称',
  `mode_id` bigint(20) NOT NULL COMMENT '标准数据模块ID',
  `mode_name` varchar(32) NOT NULL COMMENT '标准数据模块名称',
  `column_id` bigint(20) NOT NULL COMMENT '数据库表中的列信息记录ID',
  `table_cname` varchar(32) NOT NULL COMMENT '表中文名',
  `table_ename` varchar(32) NOT NULL COMMENT '表英文名',
  `column_cname` varchar(32) NOT NULL COMMENT '字段中文名',
  `column_ename` varchar(32) NOT NULL COMMENT '字段英文名',
  `table_id` bigint(20) NOT NULL COMMENT '校验字段所在表的表记录ID',
  `table_val` varchar(32) DEFAULT NULL COMMENT '表字段的值',
  `type` varchar(2) NOT NULL COMMENT '字段问题类型,1-数据缺失，2-非标准值，3-正则校验失败',
  `result_id` bigint(20) NOT NULL COMMENT '校验结果表记录ID，如：表字段的非空校验结果表、表字段的正则式校验结果表、表字段的标准值校验结果表',
  `is_solved` varchar(2) NOT NULL DEFAULT '0' COMMENT '是否已解决,0:未解决，1:已解决。机器通过比对，若数据问题已解决，则更新为已解决。',
  `solve_time` datetime DEFAULT NULL COMMENT '最近一次机器比对时间',
  `is_audited` varchar(2) NOT NULL DEFAULT '0' COMMENT '是否通过了人工核查,0:未通过，1:已通过。对于未解决的问题数据，人工进行干预，确认其是否通过校验。',
  `audit_time` datetime DEFAULT NULL COMMENT '最近一次人工核查时间',
  `description` varchar(512) DEFAULT NULL COMMENT '描述',
  `is_deleted` char(1) NOT NULL DEFAULT 'N' COMMENT '是否删除,N:未删除，Y:删除',
  `gmt_create` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录创建时间',
  `gmt_modified` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录修改时间,如果时间是1970年则表示纪录未修改',
  `creator` varchar(32) NOT NULL DEFAULT '0' COMMENT '创建人，0表示无创建人值',
  `modifier` varchar(32) NOT NULL DEFAULT '0' COMMENT '修改人,如果为0则表示纪录未修改',
  `remark` varchar(128) DEFAULT NULL COMMENT '备注',
  PRIMARY KEY (`id`),
  UNIQUE KEY `uniq_type_result_id` (`type`,`result_id`) USING BTREE COMMENT '字段type和字段result_id的值组合必须全表唯一'
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='表字段的实际值问题校验结果汇总表';

-- ----------------------------
-- Table structure for dc_column_verify
-- ----------------------------
DROP TABLE IF EXISTS `dc_column_verify`;
CREATE TABLE `dc_column_verify` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `column_id` bigint(20) NOT NULL COMMENT '数据库表中的列信息记录ID',
  `table_cname` varchar(32) NOT NULL COMMENT '表中文名',
  `table_ename` varchar(32) NOT NULL COMMENT '表英文名',
  `column_cname` varchar(32) NOT NULL COMMENT '字段中文名',
  `column_ename` varchar(32) NOT NULL COMMENT '字段英文名',
  `type` varchar(2) NOT NULL DEFAULT '0' COMMENT '校验类型,0:未知，1:数据缺失，2:标准值，3:正则',
  `verify_id` bigint(20) NOT NULL COMMENT '校验表记录ID，如：表字段的非空校验维护表、表字段的实际值与正则式关联表、表字段的标准值校验维护表',
  `verify_val` varchar(32) NOT NULL COMMENT '校验表值',
  `description` varchar(512) DEFAULT NULL COMMENT '描述',
  `is_deleted` char(1) NOT NULL DEFAULT 'N' COMMENT '是否删除,N:未删除，Y:删除',
  `gmt_create` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录创建时间',
  `gmt_modified` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录修改时间,如果时间是1970年则表示纪录未修改',
  `creator` varchar(32) NOT NULL DEFAULT '0' COMMENT '创建人，0表示无创建人值',
  `modifier` varchar(32) NOT NULL DEFAULT '0' COMMENT '修改人,如果为0则表示纪录未修改',
  `remark` varchar(128) DEFAULT NULL COMMENT '备注',
  PRIMARY KEY (`id`),
  UNIQUE KEY `uniq_type_verify_id` (`type`,`verify_id`) USING BTREE COMMENT '字段type和字段verify_id的值组合必须全表唯一'
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='表字段的实际值问题校验维护汇总表';

-- ----------------------------
-- Table structure for dc_nonnull_info
-- ----------------------------
DROP TABLE IF EXISTS `dc_nonnull_info`;
CREATE TABLE `dc_nonnull_info` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `column_id` bigint(20) NOT NULL COMMENT '数据库表中的列信息记录ID',
  `is_required` varchar(2) NOT NULL DEFAULT '1' COMMENT '是否必填,0:否，1:是',
  `is_deleted` char(1) NOT NULL DEFAULT 'N' COMMENT '是否删除,N:未删除，Y:删除',
  `gmt_create` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录创建时间',
  `gmt_modified` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录修改时间,如果时间是1970年则表示纪录未修改',
  `creator` varchar(32) NOT NULL DEFAULT '0' COMMENT '创建人，0表示无创建人值',
  `modifier` varchar(32) NOT NULL DEFAULT '0' COMMENT '修改人,如果为0则表示纪录未修改',
  `remark` varchar(128) DEFAULT NULL COMMENT '备注',
  PRIMARY KEY (`id`),
  UNIQUE KEY `uniq_column_id` (`column_id`) USING BTREE COMMENT '字段column_id的值必须全表唯一'
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='表字段的非空校验维护表';

-- ----------------------------
-- Table structure for dc_nonnull_result
-- ----------------------------
DROP TABLE IF EXISTS `dc_nonnull_result`;
CREATE TABLE `dc_nonnull_result` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `hospital_id` bigint(20) NOT NULL DEFAULT '0' COMMENT '组织机构ID',
  `behospital_code` varchar(32) NOT NULL COMMENT '病人住院序号',
  `hospital_data_code` varchar(128) NOT NULL COMMENT '医院数据编号',
  `hosptial_datatmp_code` varchar(32) DEFAULT NULL COMMENT '医院病历文书模板编号/非文书模块编号',
  `hosptial_datatmp_name` varchar(32) DEFAULT NULL COMMENT '医院病历文书模板名称/非文书模块名称',
  `mode_id` bigint(20) NOT NULL COMMENT '标准数据模块ID',
  `mode_name` varchar(32) NOT NULL COMMENT '标准数据模块名称',
  `column_id` bigint(20) NOT NULL COMMENT '数据库表中的列信息记录ID',
  `table_cname` varchar(32) NOT NULL COMMENT '表中文名',
  `table_ename` varchar(32) NOT NULL COMMENT '表英文名',
  `column_cname` varchar(32) NOT NULL COMMENT '字段中文名',
  `column_ename` varchar(32) NOT NULL COMMENT '字段英文名',
  `table_id` bigint(20) NOT NULL COMMENT '校验字段所在表的表记录ID',
  `table_val` varchar(32) DEFAULT NULL COMMENT '表字段的值',
  `is_solved` varchar(2) NOT NULL DEFAULT '0' COMMENT '是否已解决,0:未解决，1:已解决。机器通过比对，若数据问题已解决，则更新为已解决。',
  `solve_time` datetime DEFAULT NULL COMMENT '最近一次机器比对时间',
  `is_audited` varchar(2) NOT NULL DEFAULT '0' COMMENT '是否通过了人工核查,0:未通过，1:已通过。对于未解决的问题数据，人工进行干预，确认其是否通过校验。',
  `audit_time` datetime DEFAULT NULL COMMENT '最近一次人工核查时间',
  `description` varchar(512) DEFAULT NULL COMMENT '描述',
  `is_deleted` char(1) NOT NULL DEFAULT 'N' COMMENT '是否删除,N:未删除，Y:删除',
  `gmt_create` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录创建时间',
  `gmt_modified` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录修改时间,如果时间是1970年则表示纪录未修改',
  `creator` varchar(32) NOT NULL DEFAULT '0' COMMENT '创建人，0表示无创建人值',
  `modifier` varchar(32) NOT NULL DEFAULT '0' COMMENT '修改人,如果为0则表示纪录未修改',
  `remark` varchar(128) DEFAULT NULL COMMENT '备注',
  PRIMARY KEY (`id`),
  UNIQUE KEY `uniq_column_table_id` (`column_id`,`table_id`) USING BTREE COMMENT '字段column_id和字段table_id的值组合必须全表唯一，即：在当前表中，某个表的某条记录的某个字段值只能有一条校验记录。'
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='表字段的非空校验结果表';

-- ----------------------------
-- Table structure for dc_regular_info
-- ----------------------------
DROP TABLE IF EXISTS `dc_regular_info`;
CREATE TABLE `dc_regular_info` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `name` varchar(32) NOT NULL COMMENT '正则式名称',
  `val` varchar(256) NOT NULL COMMENT '正则式值',
  `description` varchar(512) NOT NULL COMMENT '描述',
  `is_deleted` char(1) NOT NULL DEFAULT 'N' COMMENT '是否删除,N:未删除，Y:删除',
  `gmt_create` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录创建时间',
  `gmt_modified` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录修改时间,如果时间是1970年则表示纪录未修改',
  `creator` varchar(32) NOT NULL DEFAULT '0' COMMENT '创建人，0表示无创建人值',
  `modifier` varchar(32) NOT NULL DEFAULT '0' COMMENT '修改人,如果为0则表示纪录未修改',
  `remark` varchar(128) DEFAULT NULL COMMENT '备注',
  PRIMARY KEY (`id`),
  UNIQUE KEY `uniq_name` (`name`) USING BTREE COMMENT '字段name的值必须全表唯一'
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='正则式维护表';

-- ----------------------------
-- Table structure for dc_regular_mapping
-- ----------------------------
DROP TABLE IF EXISTS `dc_regular_mapping`;
CREATE TABLE `dc_regular_mapping` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `column_id` bigint(20) NOT NULL COMMENT '数据库表中的列信息记录ID',
  `regular_id` bigint(20) NOT NULL COMMENT '正则式ID',
  `is_deleted` char(1) NOT NULL DEFAULT 'N' COMMENT '是否删除,N:未删除，Y:删除',
  `gmt_create` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录创建时间',
  `gmt_modified` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录修改时间,如果时间是1970年则表示纪录未修改',
  `creator` varchar(32) NOT NULL DEFAULT '0' COMMENT '创建人，0表示无创建人值',
  `modifier` varchar(32) NOT NULL DEFAULT '0' COMMENT '修改人,如果为0则表示纪录未修改',
  `remark` varchar(128) DEFAULT NULL COMMENT '备注',
  PRIMARY KEY (`id`),
  UNIQUE KEY `uniq_column_regular_id` (`column_id`,`regular_id`) USING BTREE COMMENT '字段column_id和字段regular_id的值组合必须全表唯一'
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='表字段的实际值与正则式关联表';

-- ----------------------------
-- Table structure for dc_regular_result
-- ----------------------------
DROP TABLE IF EXISTS `dc_regular_result`;
CREATE TABLE `dc_regular_result` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `hospital_id` bigint(20) NOT NULL DEFAULT '0' COMMENT '组织机构ID',
  `behospital_code` varchar(32) NOT NULL COMMENT '病人住院序号',
  `hospital_data_code` varchar(128) NOT NULL COMMENT '医院数据编号',
  `hosptial_datatmp_code` varchar(32) DEFAULT NULL COMMENT '医院病历文书模板编号/非文书模块编号',
  `hosptial_datatmp_name` varchar(32) DEFAULT NULL COMMENT '医院病历文书模板名称/非文书模块名称',
  `mode_id` bigint(20) NOT NULL COMMENT '标准数据模块ID',
  `mode_name` varchar(32) NOT NULL COMMENT '标准数据模块名称',
  `column_id` bigint(20) NOT NULL COMMENT '数据库表中的列信息记录ID',
  `table_cname` varchar(32) NOT NULL COMMENT '表中文名',
  `table_ename` varchar(32) NOT NULL COMMENT '表英文名',
  `column_cname` varchar(32) NOT NULL COMMENT '字段中文名',
  `column_ename` varchar(32) NOT NULL COMMENT '字段英文名',
  `regular_id` bigint(20) NOT NULL COMMENT '正则式ID',
  `regular_name` varchar(32) NOT NULL COMMENT '正则式名称',
  `regular_val` varchar(256) NOT NULL COMMENT '正则式值',
  `regular_des` varchar(512) NOT NULL COMMENT '正则式描述',
  `table_id` bigint(20) NOT NULL COMMENT '校验字段所在表的表记录ID',
  `table_val` varchar(32) DEFAULT NULL COMMENT '表字段的值',
  `is_solved` varchar(2) NOT NULL DEFAULT '0' COMMENT '是否已解决,0:未解决，1:已解决。机器通过比对，若数据问题已解决，则更新为已解决。',
  `solve_time` datetime DEFAULT NULL COMMENT '最近一次机器比对时间',
  `is_audited` varchar(2) NOT NULL DEFAULT '0' COMMENT '是否通过了人工核查,0:未通过，1:已通过。对于未解决的问题数据，人工进行干预，确认其是否通过校验。',
  `audit_time` datetime DEFAULT NULL COMMENT '最近一次人工核查时间',
  `description` varchar(512) DEFAULT NULL COMMENT '描述',
  `is_deleted` char(1) NOT NULL DEFAULT 'N' COMMENT '是否删除,N:未删除，Y:删除',
  `gmt_create` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录创建时间',
  `gmt_modified` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录修改时间,如果时间是1970年则表示纪录未修改',
  `creator` varchar(32) NOT NULL DEFAULT '0' COMMENT '创建人，0表示无创建人值',
  `modifier` varchar(32) NOT NULL DEFAULT '0' COMMENT '修改人,如果为0则表示纪录未修改',
  `remark` varchar(128) DEFAULT NULL COMMENT '备注',
  PRIMARY KEY (`id`),
  UNIQUE KEY `uniq_column_table_id` (`column_id`,`table_id`) USING BTREE COMMENT '字段column_id和字段table_id的值组合必须全表唯一，即：在当前表中，某个表的某条记录的某个字段值只能有一条校验记录。'
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='表字段的正则式校验结果表';

-- ----------------------------
-- Table structure for dc_standardvalue_info
-- ----------------------------
DROP TABLE IF EXISTS `dc_standardvalue_info`;
CREATE TABLE `dc_standardvalue_info` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `column_id` bigint(20) NOT NULL COMMENT '数据库表中的列信息记录ID',
  `val` varchar(32) NOT NULL COMMENT '表字段的标准值',
  `is_deleted` char(1) NOT NULL DEFAULT 'N' COMMENT '是否删除,N:未删除，Y:删除',
  `gmt_create` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录创建时间',
  `gmt_modified` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录修改时间,如果时间是1970年则表示纪录未修改',
  `creator` varchar(32) NOT NULL DEFAULT '0' COMMENT '创建人，0表示无创建人值',
  `modifier` varchar(32) NOT NULL DEFAULT '0' COMMENT '修改人,如果为0则表示纪录未修改',
  `remark` varchar(128) DEFAULT NULL COMMENT '备注',
  PRIMARY KEY (`id`),
  UNIQUE KEY `uniq_column_id_val` (`column_id`,`val`) USING BTREE COMMENT '字段column_id和字段val的值组合必须全表唯一'
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='表字段的标准值校验维护表';

-- ----------------------------
-- Table structure for dc_standardvalue_mapping
-- ----------------------------
DROP TABLE IF EXISTS `dc_standardvalue_mapping`;
CREATE TABLE `dc_standardvalue_mapping` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `standardvalue_id` bigint(20) NOT NULL COMMENT '表字段的标准值校验ID',
  `hospital_id` bigint(20) NOT NULL DEFAULT '0' COMMENT '组织机构ID',
  `hospital_val` varchar(32) NOT NULL COMMENT '表字段的实际值',
  `is_deleted` char(1) NOT NULL DEFAULT 'N' COMMENT '是否删除,N:未删除，Y:删除',
  `gmt_create` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录创建时间',
  `gmt_modified` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录修改时间,如果时间是1970年则表示纪录未修改',
  `creator` varchar(32) NOT NULL DEFAULT '0' COMMENT '创建人，0表示无创建人值',
  `modifier` varchar(32) NOT NULL DEFAULT '0' COMMENT '修改人,如果为0则表示纪录未修改',
  `remark` varchar(128) DEFAULT NULL COMMENT '备注',
  PRIMARY KEY (`id`),
  UNIQUE KEY `uniq_hos_standval` (`standardvalue_id`,`hospital_id`,`hospital_val`) USING BTREE COMMENT '字段standardvalue_id、字段hospital_id和字段hospital_val的值组合必须全表唯一'
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='表字段的实际值与标准值映射表';

-- ----------------------------
-- Table structure for dc_standardvalue_result
-- ----------------------------
DROP TABLE IF EXISTS `dc_standardvalue_result`;
CREATE TABLE `dc_standardvalue_result` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `hospital_id` bigint(20) NOT NULL DEFAULT '0' COMMENT '组织机构ID',
  `behospital_code` varchar(32) NOT NULL COMMENT '病人住院序号',
  `hospital_data_code` varchar(128) NOT NULL COMMENT '医院数据编号',
  `hosptial_datatmp_code` varchar(32) DEFAULT NULL COMMENT '医院病历文书模板编号/非文书模块编号',
  `hosptial_datatmp_name` varchar(32) DEFAULT NULL COMMENT '医院病历文书模板名称/非文书模块名称',
  `mode_id` bigint(20) NOT NULL COMMENT '标准数据模块ID',
  `mode_name` varchar(32) NOT NULL COMMENT '标准数据模块名称',
  `column_id` bigint(20) NOT NULL COMMENT '数据库表中的列信息记录ID',
  `table_cname` varchar(32) NOT NULL COMMENT '表中文名',
  `table_ename` varchar(32) NOT NULL COMMENT '表英文名',
  `column_cname` varchar(32) NOT NULL COMMENT '字段中文名',
  `column_ename` varchar(32) NOT NULL COMMENT '字段英文名',
  `mappings` varchar(512) NOT NULL COMMENT '映射对汇总，逗号隔开，比如：标准A-医院A，标准B-医院B。医院未映射的采用标准A-无。',
  `table_id` bigint(20) NOT NULL COMMENT '校验字段所在表的表记录ID',
  `table_val` varchar(32) DEFAULT NULL COMMENT '表字段的值',
  `is_solved` varchar(2) NOT NULL DEFAULT '0' COMMENT '是否已解决,0:未解决，1:已解决。机器通过比对，若数据问题已解决，则更新为已解决。',
  `solve_time` datetime DEFAULT NULL COMMENT '最近一次机器比对时间',
  `is_audited` varchar(2) NOT NULL DEFAULT '0' COMMENT '是否通过了人工核查,0:未通过，1:已通过。对于未解决的问题数据，人工进行干预，确认其是否通过校验。',
  `audit_time` datetime DEFAULT NULL COMMENT '最近一次人工核查时间',
  `description` varchar(512) DEFAULT NULL COMMENT '描述',
  `is_deleted` char(1) NOT NULL DEFAULT 'N' COMMENT '是否删除,N:未删除，Y:删除',
  `gmt_create` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录创建时间',
  `gmt_modified` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录修改时间,如果时间是1970年则表示纪录未修改',
  `creator` varchar(32) NOT NULL DEFAULT '0' COMMENT '创建人，0表示无创建人值',
  `modifier` varchar(32) NOT NULL DEFAULT '0' COMMENT '修改人,如果为0则表示纪录未修改',
  `remark` varchar(128) DEFAULT NULL COMMENT '备注',
  PRIMARY KEY (`id`),
  UNIQUE KEY `uniq_column_table_id` (`column_id`,`table_id`) USING BTREE COMMENT '字段column_id和字段table_id的值组合必须全表唯一，即：在当前表中，某个表的某条记录的某个字段值只能有一条校验记录。'
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='表字段的标准值校验结果表';

-- ----------------------------
-- Table structure for hos_disease_info
-- ----------------------------
DROP TABLE IF EXISTS `hos_disease_info`;
CREATE TABLE `hos_disease_info` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `hospital_id` bigint(20) NOT NULL DEFAULT '0' COMMENT '组织机构ID',
  `code` varchar(32) DEFAULT NULL COMMENT '医院诊断代码',
  `name` varchar(32) DEFAULT NULL COMMENT '医院诊断名称',
  `icd10` varchar(32) DEFAULT NULL COMMENT '医院诊断ICD-10编码',
  `standard` varchar(512) DEFAULT NULL COMMENT '标准词，规范：标准词名称_特性1名称:特性1值_特性2名称:特性2值',
  `synonym` varchar(512) DEFAULT NULL COMMENT '同义词',
  `is_deleted` char(1) NOT NULL DEFAULT 'N' COMMENT '是否删除,N:未删除，Y:删除',
  `gmt_create` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录创建时间',
  `gmt_modified` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录修改时间,如果时间是1970年则表示纪录未修改',
  `creator` varchar(32) NOT NULL DEFAULT '0' COMMENT '创建人，0表示无创建人值',
  `modifier` varchar(32) NOT NULL DEFAULT '0' COMMENT '修改人,如果为0则表示纪录未修改',
  `remark` varchar(128) DEFAULT NULL COMMENT '备注',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='医院诊断信息表';

-- ----------------------------
-- Table structure for hos_drug_info
-- ----------------------------
DROP TABLE IF EXISTS `hos_drug_info`;
CREATE TABLE `hos_drug_info` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `hospital_id` bigint(20) NOT NULL DEFAULT '0' COMMENT '组织机构ID',
  `code` varchar(32) DEFAULT NULL COMMENT '医院药品代码',
  `name` varchar(32) DEFAULT NULL COMMENT '医院药品名称',
  `approval_num` varchar(256) DEFAULT NULL COMMENT '国药准字',
  `standard` varchar(512) DEFAULT NULL COMMENT '标准词，规范：标准词名称_特性1名称:特性1值_特性2名称:特性2值',
  `synonym` varchar(512) DEFAULT NULL COMMENT '同义词',
  `is_deleted` char(1) NOT NULL DEFAULT 'N' COMMENT '是否删除,N:未删除，Y:删除',
  `gmt_create` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录创建时间',
  `gmt_modified` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录修改时间,如果时间是1970年则表示纪录未修改',
  `creator` varchar(32) NOT NULL DEFAULT '0' COMMENT '创建人，0表示无创建人值',
  `modifier` varchar(32) NOT NULL DEFAULT '0' COMMENT '修改人,如果为0则表示纪录未修改',
  `remark` varchar(128) DEFAULT NULL COMMENT '备注',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='医院药品信息表';

-- ----------------------------
-- Table structure for hos_official_capacity
-- ----------------------------
DROP TABLE IF EXISTS `hos_official_capacity`;
CREATE TABLE `hos_official_capacity` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `hospital_id` bigint(20) NOT NULL DEFAULT '0' COMMENT '组织机构ID',
  `name` varchar(32) DEFAULT NULL COMMENT '职务/职称名称',
  `type` varchar(2) NOT NULL DEFAULT '0' COMMENT '类型:0:未知，1:职务，2职称',
  `change_time` datetime DEFAULT NULL COMMENT '职位/职称变更时间',
  `doctor_code` varchar(32) DEFAULT NULL COMMENT '医生代码',
  `doctor_name` varchar(32) DEFAULT NULL COMMENT '医生名称',
  `dept_name` varchar(32) DEFAULT NULL COMMENT '科室名称',
  `is_deleted` char(1) NOT NULL DEFAULT 'N' COMMENT '是否删除,N:未删除，Y:删除',
  `gmt_create` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录创建时间',
  `gmt_modified` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录修改时间,如果时间是1970年则表示纪录未修改',
  `creator` varchar(32) NOT NULL DEFAULT '0' COMMENT '创建人，0表示无创建人值',
  `modifier` varchar(32) NOT NULL DEFAULT '0' COMMENT '修改人,如果为0则表示纪录未修改',
  `remark` varchar(128) DEFAULT NULL COMMENT '备注',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='医院职位职称信息表';

-- ----------------------------
-- Table structure for hos_operation_info
-- ----------------------------
DROP TABLE IF EXISTS `hos_operation_info`;
CREATE TABLE `hos_operation_info` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `hospital_id` bigint(20) NOT NULL DEFAULT '0' COMMENT '组织机构ID',
  `code` varchar(32) DEFAULT NULL COMMENT '医院手术代码',
  `name` varchar(32) DEFAULT NULL COMMENT '医院手术名称',
  `standard` varchar(512) DEFAULT NULL COMMENT '标准词，规范：标准词名称_特性1名称:特性1值_特性2名称:特性2值',
  `synonym` varchar(512) DEFAULT NULL COMMENT '同义词',
  `is_deleted` char(1) NOT NULL DEFAULT 'N' COMMENT '是否删除,N:未删除，Y:删除',
  `gmt_create` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录创建时间',
  `gmt_modified` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录修改时间,如果时间是1970年则表示纪录未修改',
  `creator` varchar(32) NOT NULL DEFAULT '0' COMMENT '创建人，0表示无创建人值',
  `modifier` varchar(32) NOT NULL DEFAULT '0' COMMENT '修改人,如果为0则表示纪录未修改',
  `remark` varchar(128) DEFAULT NULL COMMENT '备注',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='医院手术信息表';

-- ----------------------------
-- Table structure for hos_record_template
-- ----------------------------
DROP TABLE IF EXISTS `hos_record_template`;
CREATE TABLE `hos_record_template` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `hospital_id` bigint(20) NOT NULL DEFAULT '0' COMMENT '组织机构ID',
  `code` varchar(32) DEFAULT NULL COMMENT '模板代码',
  `name` varchar(32) DEFAULT NULL COMMENT '模板名称',
  `content` longtext COMMENT '模板内容',
  `type` varchar(2) NOT NULL DEFAULT '0' COMMENT '模板类型,0:未知，1:html，2:xml',
  `parent_code` varchar(32) DEFAULT NULL COMMENT '父模板代码',
  `parent_name` varchar(32) DEFAULT NULL COMMENT '父模板名称',
  `is_deleted` char(1) NOT NULL DEFAULT 'N' COMMENT '是否删除,N:未删除，Y:删除',
  `gmt_create` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录创建时间',
  `gmt_modified` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录修改时间,如果时间是1970年则表示纪录未修改',
  `creator` varchar(32) NOT NULL DEFAULT '0' COMMENT '创建人，0表示无创建人值',
  `modifier` varchar(32) NOT NULL DEFAULT '0' COMMENT '修改人,如果为0则表示纪录未修改',
  `remark` varchar(128) DEFAULT NULL COMMENT '备注',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='医院病历文书模板信息表';

-- ----------------------------
-- Table structure for base_concept_library
-- ----------------------------
DROP TABLE IF EXISTS `base_concept_library`;
CREATE TABLE `base_concept_library` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '主键',
  `standard` varchar(128) NOT NULL COMMENT '标准词',
  `synonym` varchar(128) NOT NULL COMMENT '同义词',
  `spell` varchar(128) DEFAULT NULL COMMENT '同义词的拼音首字母(大写)',
  `type` varchar(2) NOT NULL COMMENT '术语类型,1:诊断，2:手术，3:药品',
  `is_deleted` char(1) NOT NULL DEFAULT 'N' COMMENT '是否删除,N:未删除，Y:删除',
  `gmt_create` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录创建时间',
  `gmt_modified` datetime NOT NULL DEFAULT '1970-01-01 12:00:00' COMMENT '记录修改时间,如果时间是1970年则表示纪录未修改',
  `creator` varchar(32) NOT NULL DEFAULT '0' COMMENT '创建人，0表示无创建人值',
  `modifier` varchar(32) NOT NULL DEFAULT '0' COMMENT '修改人,如果为0则表示纪录未修改',
  `remark` varchar(128) DEFAULT NULL COMMENT '备注',
  PRIMARY KEY (`id`),
  UNIQUE KEY `uniq_stand_syn_type` (`standard`,`synonym`,`type`) USING BTREE COMMENT '字段standard、字段synonym和字段type的值组合必须全表唯一'
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='术语表';